"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var t = _interopRequireWildcard(require("@babel/types"));

var _babelPluginMacros = require("babel-plugin-macros");

var _evalNode = require("./babel/eval-node");

var _tokens = require("./styling/tokens");

var _addImport = require("./babel/add-import");

var _injectStyles = require("./babel/inject-styles");

var _defaultVariant = require("./styling/utils/defaultVariant");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

// Packages
// Ours
const styledMacro = ({
  references,
  state
}) => {
  var _references$default;

  const program = state.file.path; // A map that holds the key/value pairs of the generated styles.

  const styles = {}; // Variable name to be used later with StyleSheet.create

  const stylesVariableId = program.scope.generateUidIdentifier('styles');
  (_references$default = references.default) === null || _references$default === void 0 ? void 0 : _references$default.forEach(refPath => {
    if (!t.isCallExpression(refPath.parent)) {
      return;
    } // Get the call expression


    const callExpr = refPath.parent; // Parse token argument

    const {
      confident,
      value: tokens,
      deopt
    } = (0, _evalNode.evalNode)(refPath.parentPath, callExpr.arguments[0]);

    if (!confident) {
      throw deopt === null || deopt === void 0 ? void 0 : deopt.buildCodeFrameError('could not evaluate style names at compile time');
    }

    if (!Array.isArray(tokens)) {
      throw deopt === null || deopt === void 0 ? void 0 : deopt.buildCodeFrameError('style names must be an array of strings');
    } // Avoid wrapping styles around select(..) call if we only have
    // defautl variant styles.


    let shouldUseSelect = false;

    try {
      // Resolve styles
      callExpr.arguments[0] = t.arrayExpression((0, _tokens.resolveTokens)(tokens).map(style => {
        if (style.variant !== _defaultVariant.DEFAULT_VARIANT) {
          shouldUseSelect = true;
        } // Generate an Id to be used as a key in the StyleSheet
        // later.


        const styleId = program.scope.generateUidIdentifier(style.variant);
        styles[styleId.name] = style.style;
        return t.objectExpression([// => { vairant: style.variant }
        t.objectProperty(t.identifier('variant'), t.stringLiteral(style.variant)), // => { style: styles.styleId }
        t.objectProperty(t.identifier('style'), t.memberExpression(stylesVariableId, styleId))]);
      }));
    } catch (error) {
      throw refPath.buildCodeFrameError(error.message);
    }

    if (!shouldUseSelect) {
      // We know for sure there is exactly one element (if any)
      const defaultStyle = callExpr.arguments[0].elements[0]; // => { style: defaultStyle , ... }

      refPath.parentPath.replaceWith(t.objectExpression(defaultStyle ? [defaultStyle.properties[1]] : []));
      return;
    } // Import StyleUtils.select
    // => import { select } from 'path/to/util';
    //    select([...], ...)


    callExpr.callee = (0, _addImport.importUtil)(program, 'select');
  }); // Inject generated styles to the module

  if (Object.keys(styles).length > 0) {
    (0, _injectStyles.injectStyles)(program, stylesVariableId, styles);
  }
};

var _default = (0, _babelPluginMacros.createMacro)(styledMacro);

exports.default = _default;
//# sourceMappingURL=macro.js.map